/*****************************************************
 Project : Power MCU Code
 Version : 3.0.1
 Date    : 4/1/2011
 Author  : Rajesh Atluri
 Company : Cornell Violet Satellite Project
  
 Chip type           : ATmega128
 Program type        : Application
 Clock frequency     : 16.000000 MHz
 Memory model        : -
 External SRAM size  : -
 Data Stack size     : -
 *****************************************************/

#include "power.h"

 
// UART file descriptor, putchar and getchar are in uart.c 
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);
 
 /* NOTE: 	MUST MAKE SIGNIFICANT CHANGES TO SVIT ARRAY FOR ERROR HANDLING (e.g. FLAG BITS) */
// MOD: THIS svit ARRAY HAS fc5'S CURRENT ON MUX1.0 AND gps1'S CURRENT ON MUX1.1
// MOD: switches of spec and star are "d4" and "d5" respectively. fog15's switch "c1"

// SVIT array, currently 36-elements of SVIT_t structs
SVIT_t SVIT[] = {
{"spec",   	"d4", 0, 0x00, /*{0,256,0,256},*/  1, 16, 0, 1,   1,  6, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"star",   	"d5", 0, 0x00, /*{0,256,0,256},*/  1, 23, 0, 1,   1,  7, 0, 1,   2,  7, 0},
{"fc5",    	"a0", 0, 0x00, /*{0,256,0,256},*/  1, 19, 0, 1,   1,  0, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"fc3.3",  	"a0", 0, 0x00, /*{0,256,0,256},*/  2, 13, 0, 1,   2, 14, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"gps1",   	"a1", 0, 0x00, /*{0,256,0,256},*/  1, 21, 0, 1,   1,  1, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"gps2",  	"a2", 0, 0x00, /*{0,256,0,256},*/  1, 26, 0, 1,   1,  2, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"ib",     	"b5", 0, 0x00, /*{0,256,0,256},*/  1, 29, 0, 1,   1,  3, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"heat1",  	"b6", 0, 0x00, /*{0,256,0,256},*/  1, 14, 0, 1,   1, 31, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"heat2",  	"b7", 0, 0x00, /*{0,256,0,256},*/  1, 27, 0, 1,   1,  4, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"cmg",    	"g2", 0, 0x00, /*{0,256,0,256},*/  1, 15, 0, 1,   1,  8, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"sunsen", 	"c7", 0, 0x00, /*{0,256,0,256},*/  1, 25, 0, 1,   1,  9, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"radio1", 	"c6", 0, 0x00, /*{0,256,0,256},*/  1, 24, 0, 1,   1, 10, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"radio2", 	"c5", 0, 0x00, /*{0,256,0,256},*/  1, 17, 0, 1,   1, 11, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"maestro", "c4", 0, 0x00, /*{0,256,0,256},*/  1, 22, 0, 1,   1, 12, 0, 1,   2,  8, 0},
{"mag", 	"c3", 0, 0x00, /*{0,256,0,256},*/  0,  4, 0, 1,   0,  6, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"fog15", 	"c1", 0, 0x00, /*{0,256,0,256},*/  1, 18, 0, 1,   1, 13, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"fog5",	"c2", 0, 0x00, /*{0,256,0,256},*/  1, 18, 0, 1,   1, 13, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"torq1", 	"",   0, 0x00, /*{0,256,0,256},*/  1, 28, 0, 1,   1,  5, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"torq2", 	"",   0, 0x00, /*{0,256,0,256},*/  2, 15, 0, 1,   2, 16, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"torq3", 	"",   0, 0x00, /*{0,256,0,256},*/  2, 17, 0, 1,   2, 18, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"batt1",	"",   0, 0x00, /*{0,256,0,256},*/  0,  1, 0, 1,   0,  2, 0, 1,   2,  9, 0}, // says 9, 10 for Bit on SVIT.xlsx; average them
{"batt2", 	"",   0, 0x00, /*{0,256,0,256},*/  1, 20, 0, 1,   1, 30, 0, 1,   2, 11, 0}, // says 11, 12 for Bit on SVIT.xlsx; average them
{"solar", 	"",	  0, 0x00, /*{0,256,0,256},*/  0,  7, 0, 1,   0,  3, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"solar1",	"",   0, 0x00, /*{0,256,0,256},*/  0, 18, 0, 1,   0, 20, 0, 1,   2,  0, 0},
{"solar2",  "",   0, 0x00, /*{0,256,0,256},*/  0, 19, 0, 1,   0, 21, 0, 1,   2,  1, 0},
{"solar3",  "",   0, 0x00, /*{0,256,0,256},*/  0, 10, 0, 1,   0, 22, 0, 1,   2,  2, 0},
{"solar4",  "",   0, 0x00, /*{0,256,0,256},*/  0, 15, 0, 1,   0, 23, 0, 1,   2,  3, 0},
{"solar5",  "",	  0, 0x00, /*{0,256,0,256},*/  0,  9, 0, 1,   0, 24, 0, 1,   2,  4, 0},
{"solar6",  "",   0, 0x00, /*{0,256,0,256},*/  0, 14, 0, 1,   0, 25, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"solar7",  "",   0, 0x00, /*{0,256,0,256},*/  0, 16, 0, 1,   0, 26, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"solar8",  "",   0, 0x00, /*{0,256,0,256},*/  0,  8, 0, 1,   0, 27, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"solar9",  "",   0, 0x00, /*{0,256,0,256},*/  0, 13, 0, 1,   0, 28, 0, 1,   2,  5, 0},
{"solar10", "",   0, 0x00, /*{0,256,0,256},*/  0, 12, 0, 1,   0, 29, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"solar11", "",   0, 0x00, /*{0,256,0,256},*/  0, 11, 0, 1,   0, 30, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"solar12", "",   0, 0x00, /*{0,256,0,256},*/  0, 17, 0, 1,   0, 31, 0, 1,   MUX_NULL, MUXBIT_NULL, 0},
{"pb", 		"",   0, 0x00, /*{0,256,0,256},*/  0,  5, 0, 1,   0,  0, 0, 1,   2,  6, 0}					};


 // TIMER 0 ISR
 ISR (TIMER0_COMP_vect)
 {
	if (time1 > 0)	--time1;
 }
 
 // Timer 1 ISR
 ISR (TIMER1_COMPA_vect)
 {
	if (time2 > 0)  --time2;
	if (time3 > 0)  --time3;
	if (time4 > 0)  --time4;
	if (time5 > 0)	--time5;
 }
 
 
 // PROGRAM'S MAIN METHOD
 int main(void)
 {
	initialize();
	
	// TASK SCHEDULING INFINITE WHILE LOOP
	while(1)
	{
		if (time1 == 0)
		{
			time1 = TIME_SENSE;
			read_VIT();
			storeValue();
		}
		if (time2 == 0)
		{
			time2 = TIME_COMM;
			vcp_comm();
			
			//uart_read0_term(); FOR READING STRINGS TO HYPERTERM/TERMINAL
		}
		if (time3 == 0)
		{
			time3 = TIME_WRITE;
			//vcp_write();
			
			//uart_write0_term(); FOR WRITING STRINGS TO HYPERTERM/TERMINAL
			//uart_write1();
		}
		if (time4 == 0)
		{
			time4 = TIME_UPDATE_SOC;
			// UNDEVELOPED FUNCTIONS
			//updateSOC(&BATTERY0);
			//updateSOC(&BATTERY1);
			
			// de-package the data and call right function
			//RAS_package(&PWR2, I_SAMPLE, V_SAMPLE, T_SAMPLE);
			//UCSR1B |= (1 << UDRIE1);	// start TX ISR
			//t_index1 = 0;
		}
		if (time5 == 0)
		{
			time5 = TIME_SWITCH;
			if (endSwitchState == 2)
			{
				if (SVIT[componentNum].S)
					switchControl(componentNum, 0);
				endSwitchState = 1;
			}
			//else if (SVIT[componentNum].S != endSwitchState)
				switchControl(componentNum, endSwitchState);
		}
		torqueControl(t1value, t2value, t3value);
	}
 }
  
 void initialize(void)
 {
	cli();
 
	// initialize input pins and output port registers
	DDRA = 	0xff;
	PORTA = 0x00;
	DDRB = 	0xff;
	PORTB = 0x00;
	DDRC = 	0xff;
	PORTC = 0x00;
	DDRD = 	0b11111011;
	PORTD = 0b11000000;
	DDRE = 	0b11111110;
	PORTE = 0b00000000;
	DDRF =	0xf0;
	PORTF = 0x00;
	DDRG =	0x1f;
	PORTG = 0x00;
	
	// set up uart
	uart_init();
	stdout = stdin = stderr = &uart_str;
	//fprintf(stdout, "%x%x%x", 0xC0, 0xFF, 0xC0);
		
	// TIMERS AND ADC initialization
	
	/*********************** T I M E R S ***********************/
	/*	Timer 0 initialization
	*	Clock source: External 16 MHz clock
	*	Mode: Normal top = CTC (Clear Timer on Compare)
	*	COMP value = 16
	*	Prescalar = 1
	*	Compare Match Interrupt Period = 1 us
	*/
	//TCNT0 = 0x00;
	//ASSR = (1 << AS0);	// External Clock on
	TIMSK = (1 << OCIE0);	// Enable CTC interrupt
	OCR0 = CONST_TIMER0_COMPA;	//set the compare reg. to 16 timer ticks
	TCCR0 = (1 << WGM01) | (1 << WGM00) | (1 << CS00);	// start timer at Fcpu/1
	
	/*	Timer 1 initialization
	*	Clock source: External 16 MHz clock
	*	Mode: Normal top = CTC (Clear Timer on Compare)
	*	COMPA value = 249
	*	Prescalar = 1/64
	*	COMPA Match Interrupt Period = 1 ms approx.
	*/
	TIMSK |= (1 << OCIE1A);	// Enable CTC interrupt
	OCR1A = CONST_TIMER1_COMPA;	// set the compare reg. to 250 timer ticks
	TCCR1B = (1 << WGM12) | (1 << CS11) | (1 << CS10);	// start timer at Fcpu/64
	/*********************** T I M E R S ***********************/
	
	/*********************** ADC0-ADC2 initialization ***********************/
	//ADMUX = (1 << REFS1) | (1 << REFS0) | (0 << ADLAR);	// ADC ref of 2.56V, Right justified values
	ADCSRA = ((1 << ADEN) | (1 << ADSC)) + 7;	// enable ADC, start first conversion (takes longer than others), 
												// and make ADC clock 16E6/128 = 125 kHZ
	
	index_svit = 0;
	index_temp = 0;
	
	// ASSUME BATTERY0 and BATTERY1 are 100% charged initially
	BATTERY0.SOC = 0xffffffff;
	BATTERY1.SOC = 0xffffffff;
	
	// initalize VCP pointer buffer
	uint8 BUFFER0[PWR0_BUF_SIZE];
	vcpptr_init(&PWR0, BUFFER0, PWR0_BUF_SIZE);
	// set up circular buffer state variables
	tx_in = 0;
	tx_out = 0;
	rx0_index = 0;
	tx0_index = 0;
	rx0_ready = 0;	// no (RX buffer) input ready initially
	tx0_ready = 1;	// the (TX buffer) output and the transmitter are ready initially
	vcp_newACK = 0;	// initialize ACK handshake to its RX-state value
	vcp_newTEL = 0; // initialize TEL handshake to its RX-state value
	
	// initialize the task timers/counters
	time1 = TIME_SENSE;
	time2 = TIME_COMM;
	time3 = TIME_WRITE;
	time4 = TIME_UPDATE_SOC;
	time5 = TIME_SWITCH;
	
	sei();	// crank up the ISRs
	
	getPacket(); // initialize comm. service to receive packets
 }
