/*********************** INCLUDED LIBRARIES **************************/
#include <stdio.h>
#include <inttypes.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <avr/pgmspace.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "uart.h"
#include <stdint.h>
#include "vcp/vcplib.h"
#include "vcp/common.h"


/********************************* CONSTANTS **************************/
#define CLK          16000000L
#define CONST_TIMER0_COMPA  187
#define CONST_TIMER1_COMPA  249
#define TIME_CONST   64*CONST_COMPA*TIME_SENSE/CLK  //scalar by 64, 249 COMPA, 16 MHZ

#define TIME_SENSE 		249  
#define TIME_UPDATE_SOC	499
#define TIME_COMM 		9 
#define TIME_WRITE 		9
#define TIME_SWITCH		49

#define MUX_NULL	3
#define MUXBIT_NULL	32
#define SVIT_SIZE 	36
#define TEMP_SIZE	12
#define INDEX_BATT1 20
#define INDEX_BATT2 21

#define READ(U, N) ((U) >> (N) & 1u)
#define SET(U, N) ((void)((U) |= 1u << (N)))
#define CLR(U, N) ((void)((U) &= ~(1u << (N))))
#define FLIP(U, N) ((void)((U) ^= 1u << (N)))

#define CMD_BYTE_INDEX 2
#define COMP_BYTE_INDEX 3
#define TQ1_BYTE_INDEX 3
#define FEND_B_COUNT 2
#define ADDR_B_COUNT 1
#define CRC_B_COUNT 2
#define WRAPPER_BYTE_COUNT (FEND_B_COUNT + ADDR_B_COUNT + CRC_B_COUNT)
#define PWR0_BUF_SIZE 128

//#include "v2temp.h"
//#include <avr/pgmspace.h>
// used 22Kohms
#define starting_index 	12
#define ending_index 	990
#define len				ending_index - starting_index  + 1
/**********************************************************************/


// SAMPLING STRUCTURE DEFINITION
typedef struct COMPONENT_struct
{
	unsigned char 	S_INDEX;
	unsigned short	SAMPLE[8];
} COMPONENT_t;

// BOUNDS STRUCTURE DEFINITION
typedef struct BOUNDS_struct
{
	const int LowerBound;
	const int UpperBound;
	const int AbsLowerBound;
	const int AbsUpperBound;
} BOUNDS_t;

// SENSOR COMPONENT STRUCTURE DEFINITION
typedef struct SVIT_struct
{
	char name[8];		// the shorthand subsystem name as shown in the SVIT spreadsheet
	char switchNum[3];	// the port pin (lowercase) tied to this subsystem's switch
	char S;				// the current state of the switch (1 = on, 0 = off)
	char error;			// the byte used to encode the detected errors for this subsystem/component
	
	char	Vmux;		// which analogMUX/PORTF pin is this signal on?
	char	VmuxBit;	// which of the analogMUX's inputs is this signal?
	int		V;			// most recently stored value for this voltage
	int		ScaleFactorV;	// conversion facter for this voltage signal
	//BOUNDS_t	BoundsV;
	
	char	Imux;		// which analogMUX/PORTF pin is this signal on?
	char	ImuxBit;	// which of the analogMUX's inputs is this signal?
	int		I;			// most recently stored value for this current
	int		ScaleFactorI;	// conversion factor for this current signal
	//BOUNDS_t	BoundsI;
	
	char	Tmux;		// which analogMUX/PORTF pin is this signal on?
	char	TmuxBit;	// which of the analogMUX's inputs is this signal?
	int		T;			// most recently stored value for this temperature
	//BOUNDS_t	BoundsT;
} SVIT_t;

// BATTERT STATE-OF-CHARGE STRUCTURE DEFINITION
typedef struct BATTERY_struct
{
	COMPONENT_t  	METHOD0;
	COMPONENT_t  	METHOD1;
	unsigned long	SOC;
	COMPONENT_t  	TEMP_SAMPLE;
	int				TEMPERATURE;
} BATTERY_t;


//unsigned int currentValue, voltageValue, tempValue;
COMPONENT_t I_SAMPLE[SVIT_SIZE]; // number of components and 8 samples each!
COMPONENT_t V_SAMPLE[SVIT_SIZE]; // number of components and 8 samples each!
COMPONENT_t T_SAMPLE[TEMP_SIZE]; // number of components and 8 samples each!
SVIT_t SVIT[SVIT_SIZE]; // two-dimensional SVIT array declaration
BATTERY_t BATTERY0, BATTERY1; // two batteries' SOC struct declarations


// POWER.C function declarations
void initialize(void);

// POWER_SAMPLE.C function declarations
void read_VIT(void);
unsigned int read_VIT_helper(char MUX_NUM, char MUX_SEL);
void selectMUX(char MUX_NUM, char MUX_SEL);
void storeValue(void);
int setValue(COMPONENT_t * c, unsigned int v, char ivtFlag);
void updateSOC(BATTERY_t * batt);
unsigned int filter(int sample, char SVITindex);
int getTemp(int voltage);

// POWER_CONTROL.C function declarations
void switchControl(int num, char onOffFlag);
void torqueControl(char torq1Input, char torq2Input, char torq3Input);
void drive_torq1(char state);
void drive_torq2(char state);
void drive_torq3(char state);

// POWER_COMM.C function declarations
void uart_read0_term(void);
void uart_write0_term(void);
void vcp_comm(void);
void vcp_write(void);
void putPacket(void);
void getPacket(void);
char parseMessage();
void writeACKMessage(void);
void sendTelemetry(void);
void sendSwitchStatus(void);
void sendErrors(void);
void sendBeacon(void);
void writeTELMessage(void);
void clearTxBuffer(void);

volatile int time1, time2, time3, time4, time5; //task scheduling timeout counters

vcp_ptrbuffer PWR0;	// pointer to vcp buffer used for RXing/TXing VCP packet over UART

// USART0 ISR variables
//		RXC ISR variables
volatile char rx0_status;	// return value of vcpptr_rx function
volatile unsigned int rx0_index;	// current string index
volatile char rx0_buff[20];	// input string
volatile char rx0_ready;	// flag for receive done
volatile char rx0_char;	// current character
//		TX ISR variables
volatile char tx0_status; 	// return value of vcpptr_tx function
volatile unsigned int tx_in, tx_out;	// 
volatile unsigned int tx0_index;	// current string index
volatile char tx0_buff[PWR0_BUF_SIZE];	// output string
volatile char tx0_ready;	// flag for transmit done
volatile char tx0_char;	// current character
char vcp_newACK;	// vcp_read-->vcp_write handshake
char vcp_newTEL;	// flag to indicate new Telemetry/Errors/SwitchStatus message is ready to TX

char* telMessage;	// pointer to memory location of start of TEL packet message
int v;
char ack, command; // acknowledgement byte, command byte
int index_svit, index_temp;	// index for the SVIT array, index for the T_SAMPLE array
int componentNum; // SVIT index of the component to be controlled by switch_control in while(1) loop
char endSwitchState; // final state of switch being controlled
char t1value, t2value, t3value; // input values for torque_control of the 3 torquers
