#include "power.h"

char newPacketLength;
int telSentCount;

// UART 8-bit-character-ready receive ISR
ISR(USART0_RX_vect)
{
	rx0_char = UDR0; // get an 8-bit char
	rx0_status = vcpptr_rx(&PWR0, rx0_char);
	rx0_buff[rx0_index++] = rx0_char;
	if ((rx0_status & VCP_TERM) == VCP_TERM)
	{
		UCSR0B ^= (1 << RXCIE0); // clear RX enable
		rx0_ready = 1;	// receive done
	}
}

// UART 8-bit-character-ready transmit ISR
ISR(USART0_UDRE_vect)
{
	if (tx_in == tx_out)
		UCSR0B &= ~(1 << UDRIE0); // disable TX interrupt
	else
	{
		//loop_until_bit_is_set(UCSR0A, UDRE0);
		UDR0 = tx0_char = (&PWR0)->message[tx_out];
		tx_out++;
		if (tx_out == PWR0_BUF_SIZE)
			tx_out = 0;
		/*if (tx_out >= newPacketLength)
		{
			UCSR0B ^= (1 << UDRIE0); // clear TX enable
			tx0_ready = 1;	// transmit done
		}*/
	}
}

/* 	Writes the input byte into the UART using a circular buffer approach
	and activates the UART transmit ISR	*/
int writeByte(char c)
{
	char i = tx_in;
	i++;
	if (i == PWR0_BUF_SIZE)
		i = 0;
	(&PWR0)->message[tx_in] = c;
	
	while (i == tx_out);	// until at least 1 byte free
		// tx_out modified by interrupt!
	tx_in = i;
	UCSR0B |= (1 << UDRIE0);
	//UCSR0A |= (1 << UDRE0);
	loop_until_bit_is_set(UCSR0A, UDRE0);
	
	return 0;
}

/*	Writes the message in tx0_buff to the message field of the vcpptr_buffer
	and sends the message one byte at a time	*/
void putPacket(void)
{
	tx0_ready = 0; // mark tranmitter as busy
	tx0_index = 0;
	vcpptr_clear(&PWR0);
	(&PWR0)->address = VCP_POWER;	// packets from Power MCU have VCP_POWER address (see vcplib.h)
		
	while (tx0_index < newPacketLength - WRAPPER_BYTE_COUNT)
	{	
		tx0_status = vcpptr_tx(&PWR0, tx0_buff[tx0_index++], (&PWR0)->flags);
	}
	tx0_status = vcpptr_tx(&PWR0, tx0_buff[tx0_index++], VCP_TERM);
	
	int tx_out_init = tx_out;
	while (tx_out - tx_out_init < newPacketLength)
	{
		writeByte((&PWR0)->message[tx_out]);
	}

	UCSR0B ^= (1 << UDRIE0); // clear TX enable
	tx0_ready = 1;	// transmit done
	clearTxBuffer();
	
}

/* writes an acknowledgement message into the tx0_buff buffer */
void writeACKMessage(void)
{
	newPacketLength = (char)((&PWR0)->index) + WRAPPER_BYTE_COUNT + 1;
	tx0_buff[0] = (char)(newPacketLength >> 8);
	tx0_buff[1] = (char)newPacketLength;
	tx0_buff[2] = ack;
	int j;
	for (j = 0; j < (&PWR0)->index - 2; j++)
	{
		tx0_buff[j+3] = ((&PWR0)->message[j+2]);
	}
}

/* writes the message in the telMessage buffer into the tx0_buff buffer */
void writeTELMessage(void)
{
	
	if (sizeof(&telMessage) + WRAPPER_BYTE_COUNT > PWR0_BUF_SIZE)
	{
		
	}
	else
	{
		char dataByteCount = sizeof(&telMessage) + WRAPPER_BYTE_COUNT;
		tx0_buff[0] = (char)(dataByteCount >> 8);
		tx0_buff[1] = (char)dataByteCount;
		int j;
		for (j = 0; j < (dataByteCount - WRAPPER_BYTE_COUNT); j++)
		{
			tx0_buff[j+2] = telMessage[j];
		}
		telSentCount = sizeof(&telMessage);
	}
	newPacketLength = telSentCount + WRAPPER_BYTE_COUNT;
}

// TOP-LEVEL FUNCTION OF THE VCP-PACKET-BASED COMMUNICATION OVER UART
void vcp_comm(void)
{
	if (rx0_ready)	// is the RX buffer string ready to read?
	{
		// extract message bytes from rx0_buff
		ack = parseMessage();
		getPacket(); // set up procedure to get the next string input and read it using RX ISR
		vcp_newACK = 1; // signal that a new command has been received
	}
	else if (tx0_ready && vcp_newACK)	// see if the last transmit is done and if there is a new ACK message to TX
	{
		writeACKMessage();
		//vcp_package((&PWR0)->message, &((&PWR0)->size), (&PWR0)->address, (uint8ptr)tx0_buff, sizeof(tx0_buff));
		putPacket(); // send ACK packet using transmit ISR
		
		vcp_newACK = 0; // clear the handshake from the above state
		vcpptr_clear(&PWR0);
	}
	else if (tx0_ready && vcp_newTEL && !vcp_newACK)	// see if the last transmit is done and if there is a new TEL message to TX
	{
		telSentCount = 0;
		do {
			writeTELMessage();
		} while (telSentCount < sizeof(&telMessage));
		putPacket();
		vcp_newTEL = 0;
		vcpptr_clear(&PWR0);
	}
	/*else if (tx0_ready == rx0_ready)
	{
		tx0_ready = 0;
		rx0_ready = 0;
		UCSR0B |= (1 << RXCIE0); // turn on receive ISR
	}*/
}

/* adjusts variables to enable RXing of VCP packet and enables RX ISR */
void getPacket(void)
{
	rx0_ready = 0; // mark RX buffer as not ready
	rx0_index = 0; // reset index
	UCSR0B |= (1 << RXCIE); // turn on receive ISR
}

/* breaks down PWR0's message buffer into certain fields and executes the sent command */
char parseMessage(void)
{
	char cmdStatus = 0;
	command = (&PWR0)->message[CMD_BYTE_INDEX];
	if (command <= 0x03)
		componentNum = (&PWR0)->message[COMP_BYTE_INDEX]; //componentNum = (int)msg[COMP_BYTE_INDEX];//componentNum = ((&PWR0)->message[COMP_BYTE_INDEX]);
	else 
		componentNum = 999;
	//char t1value, t2value, t3value;
	switch(command)
	{
		case 0x00:	//"CS0" - SWITCH OFF
			endSwitchState = 0;
			cmdStatus = 0x00;
			break;
		case 0x01:	//"CS1" - SWITCH ON
			endSwitchState = 1;
			cmdStatus = 0x00;
			break;
		case 0x02:	//"CST" - SWITCH RESET
			endSwitchState = 2;
			cmdStatus = 0x00;
			break;
		case 0x03:	//"CSF" - FORCE SWITCH ON
			endSwitchState = 1;
			cmdStatus = 0x00;
			break;
		case 0x04:	//"CT" - TORQUE CONTROL
			t1value = (&PWR0)->message[TQ1_BYTE_INDEX];
			t2value = (&PWR0)->message[TQ1_BYTE_INDEX + 1];
			t3value = (&PWR0)->message[TQ1_BYTE_INDEX + 2];
			//torqueControl(t1value, t2value, t3value);
			break;
		case 0x05:	//"BE" - BEACON
			
			break;
		case 0x06:	//"RTP" - SEND TELEMETRY PACKET
			sendTelemetry();
			break;
		case 0x07:	//"RSS" - REPORT SWITCH STATUS
			sendSwitchStatus();
			break;
		case 0x08:	//"RES" - REPORT ERROR STATUS
			sendErrors();
			break;
		default:
			cmdStatus = 0xff;
			break;
	}
	return cmdStatus;
}


// formats and writes a telemetry packet with data from the SVIT struct into the telMessage buffer
void sendTelemetry(void)
{
	//unsigned char vcpTXflags;
	int i;
	for (i = 0; i < SVIT_SIZE; i++)
	{
		sprintf(telMessage, "%s,%s,%x,%x\t%d\t%d\t%d\n\r", 
			SVIT[i].name, SVIT[i].switchNum, SVIT[i].S, SVIT[i].error, SVIT[i].V, SVIT[i].I, SVIT[i].T);
	}
	vcp_newTEL = 1;
}

// formats and writes the status of switches into the telMessage buffer
void sendSwitchStatus(void)
{
	int i;
	for (i = 0; i < SVIT_SIZE; i++)
	{
		sprintf(telMessage, "%x", SVIT[i].S);
		if (i < SVIT_SIZE - 1)
			sprintf(telMessage, ",");
	}
	
	vcp_newTEL = 1;
}

// formats and writes the components' error bytes into the telMessage buffer
void sendErrors(void)
{
	int i;
	for (i = 0; i < SVIT_SIZE; i++)
	{
		sprintf(telMessage, "%x", SVIT[i].error);
		if (i < SVIT_SIZE - 1)
			sprintf(telMessage, ",");
	}
		
	vcp_newTEL = 1;
}

// AS OF NOW THE BEACON FUNCTION HAS YET TO BE DEFINED
void sendBeacon(void)
{

}

// clears the tx0_buff buffer so it can get a new message into it
void clearTxBuffer(void)
{
	int i;
	for (i = 0; i <= newPacketLength; i++)
	{
		tx0_buff[i] = 0;
	}
}
