#include "power.h"

#define TORQUER_FREE 0
#define TORQUER_BRAKE 1
#define TORQUER_SETA 2
#define TORQUER_SETB 3

// NOTE TO SELF: try using "#define paste(front, back) front ## back" to reduce the number
// of lines taken up by the switchControl function

/* 	function that turns on/off the switch with index of num (orig. int, now char) 
	in the SVIT struct array. The outer if statement is included to make sure the 
	port pin is only changed if its current state differs from the final state 
	indicated by onOffFlag. (turns on if onOffFlag = 1, turns off if onOffFlag = 0) */
void switchControl(int num, char onOffFlag)
{
	char switchFound = 1;
	//if (SVIT[num].S != onOffFlag) {
		switch (SVIT[num].switchNum[0]) {
			case 'a':
				switch (SVIT[num].switchNum[1]) {
					case '2':
						if (onOffFlag) SET(PORTA, PA2);
						else CLR(PORTA, PA2);
						break;
					case '1':
						if (onOffFlag) SET(PORTA, PA1);
						else CLR(PORTA, PA1);
						break;
					case '0':
						if (onOffFlag) SET(PORTA, PA0);
						else CLR(PORTA, PA0);
						break;
					default:
						switchFound = 0;
						break;
				}
				break;
			case 'b':
				switch (SVIT[num].switchNum[1]) {
					case '7':
						if (onOffFlag) SET(PORTB, PB7);
						else CLR(PORTB, PB7);
						break;
					case '6':
						if (onOffFlag) SET(PORTB, PB6);
						else CLR(PORTB, PB6);
						break;
					case '5':
						if (onOffFlag) SET(PORTB, PB5);
						else CLR(PORTB, PB5);
						break;
					default:
						switchFound = 0;
						break;
				}
				break;
			case 'c':
				switch (SVIT[num].switchNum[1]) {
					case '7':
						if (onOffFlag) SET(PORTC, PC7);
						else CLR(PORTC, PC7);
						break;
					case '6':
						if (onOffFlag) SET(PORTC, PC6);
						else CLR(PORTC, PC6);
						break;
					case '5':
						if (onOffFlag) SET(PORTC, PC5);
						else CLR(PORTC, PC5);
						break;
					case '4':
						if (onOffFlag) SET(PORTC, PC4);
						else CLR(PORTC, PC4);
						break;
					case '3':
						if (onOffFlag) SET(PORTC, PC3);
						else CLR(PORTC, PC3);
						break;
					case '2':
						if (onOffFlag) SET(PORTC, PC2);
						else CLR(PORTC, PC2);
						break;
					case '1':
						if (onOffFlag) SET(PORTC, PC1);
						else CLR(PORTC, PC1);
					default:
						switchFound = 0;
						break;
				}
				break;
			case 'd':
				switch (SVIT[num].switchNum[1]) {
					case '5':
						if (onOffFlag) SET(PORTD, PD5);
						else CLR(PORTD, PD5);
						break;
					case '4':
						if (onOffFlag) SET(PORTD, PD4);
						else CLR(PORTD, PD4);
						break;
					default:
						switchFound = 0;
						break;
				}
				break;
			case 'g':
				switch (SVIT[num].switchNum[1]) {
					case '2':
						if (onOffFlag) SET(PORTG, PG2);
						else CLR(PORTG, PG2);
						break;
					default:
						switchFound = 0;
						break;
				}
				break;
			default:
				switchFound = 0;
				break;
		}
		if (switchFound) {
			SVIT[num].S = onOffFlag;
		}
	//}
}


// NOTE: Does torqueControl require a timer to be set into PWM mode or just a task timer?
unsigned char t1_pwm_counter, t2_pwm_counter, t3_pwm_counter; // initialize to 0 (counts 0 to 0x7f)
unsigned char t1_pwm_on, t2_pwm_on, t3_pwm_on;	// initialize to +0

/* function that controls the magnetic torquer using software-counter-based PWM */
void torqueControl(char torq1Input, char torq2Input, char torq3Input)
{
	t1_pwm_counter++;
	t2_pwm_counter++;
	t3_pwm_counter++;
	if ((t1_pwm_counter & 0x7f) > 127) { t1_pwm_counter &= 0x80; }
	if ((t2_pwm_counter & 0x7f) > 127) { t2_pwm_counter &= 0x80; }
	if ((t3_pwm_counter & 0x7f) > 127) { t3_pwm_counter &= 0x80; }
	/* SEMAPHORE LOCK */
	if (t1_pwm_counter < torq1Input) {
		if ((t1_pwm_on & 0x80) != (torq1Input & 0x80)) 
		{
			drive_torq1(TORQUER_BRAKE);
			//delay_us(10);
		}
		if (torq1Input & 0x80)	drive_torq1(TORQUER_SETA);
		else	drive_torq1(TORQUER_SETB);
	} else drive_torq1(TORQUER_FREE);
	if (t2_pwm_counter < torq2Input) {
		if ((t2_pwm_on & 0x80) != (torq2Input & 0x80)) 
		{
			drive_torq2(TORQUER_BRAKE);
			//delay_us(10);
		}
		if (torq2Input & 0x80)	drive_torq2(TORQUER_SETA);
		else	drive_torq2(TORQUER_SETB);
	} else drive_torq2(TORQUER_FREE);
	if (t3_pwm_counter < torq3Input) {
		if ((t3_pwm_on & 0x80) != (torq3Input & 0x80)) 
		{
			drive_torq3(TORQUER_BRAKE);
			//delay_us(10);
		}
		if (torq3Input & 0x80)	drive_torq3(TORQUER_SETA);
		else	drive_torq3(TORQUER_SETB);
	} else drive_torq3(TORQUER_FREE);
	t1_pwm_on = torq1Input;
	t2_pwm_on = torq2Input;
	t3_pwm_on = torq3Input;
	/* SEMAPHORE UNLOCK */
}
/* 	the following 3 functions set the output ports connected to 
	each of the 3 torquer's H-bridge interface chips	*/
void drive_torq1(char state)
{
	if (state == TORQUER_FREE) {
		SET(PORTC, PC0);
		SET(PORTG, PG1);
	} else if (state == TORQUER_BRAKE) {
		CLR(PORTC, PC0);
		CLR(PORTG, PG1);
	} else if (state == TORQUER_SETA) {
		SET(PORTC, PC0);
		CLR(PORTG, PG1);		
	} else if (state == TORQUER_SETB) {
		CLR(PORTC, PC0);
		SET(PORTG, PG1);
	}
}
void drive_torq2(char state)
{
	if (state == TORQUER_FREE) {
		SET(PORTG, PG0);
		SET(PORTD, PD0);
	} else if (state == TORQUER_BRAKE) {
		CLR(PORTG, PG0);
		CLR(PORTD, PD0);
	} else if (state == TORQUER_SETA) {
		SET(PORTG, PG0);
		CLR(PORTD, PD0);		
	} else if (state == TORQUER_SETB) {
		CLR(PORTG, PG0);
		SET(PORTD, PD0);
	}
}
void drive_torq3(char state)
{
	if (state == TORQUER_FREE) {
		SET(PORTG, PG3);
		SET(PORTG, PG4);
	} else if (state == TORQUER_BRAKE) {
		CLR(PORTG, PG3);
		CLR(PORTG, PG4);
	} else if (state == TORQUER_SETA) {
		SET(PORTG, PG3);
		CLR(PORTG, PG4);		
	} else if (state == TORQUER_SETB) {
		CLR(PORTG, PG3);
		SET(PORTG, PG4);
	}
}
