#include "power.h"

const int TempScaleFactor =	1;	// conversion factor for all temperature signals

// variables that store the digital value output by the MCU's A-to-D Converter (ADC)
unsigned int currentValue, voltageValue, tempValue;
unsigned int batt1CurrentValue, batt1VoltageValue, batt1TempValue;
unsigned int batt2CurrentValue, batt2VoltageValue, batt2TempValue;
// variables that equal the ADC output value multiplied by the appropriate sensor scaling factor
unsigned int currentScaledValue, voltageScaledValue, tempScaledValue;
unsigned int batt1CurrentScaled, batt1VoltageScaled, batt1TempScaled;
unsigned int batt2CurrentScaled, batt2VoltageScaled, batt2TempScaled;

/*	TOP LEVEL FUNCTION THAT EXECUTES AN ANALOG-TO-DIGITAL CONVERSION OF THE
	MEASURED OUTPUT OF THE SESNSORS OF THE CURRENTLY INDEXED SVIT COMPONENT 
	AND OF THE BATTERIES	*/
void read_VIT(void)
{
	/*
	*	We will use 10-bit precision, right justified (ADLAR = 0)
	*	PINF0/ADC0 = MUX with output VOLT_SEN_OUT
	*	PINF1/ADC1 = MUX with output CUR_SEN_OUT
	*	PINF2/ADC2 = MUX with output SEN_THERM_OUT
	*/
	if (index_svit++ >= SVIT_SIZE)
		index_svit = 0;
	if (index_svit == INDEX_BATT1)
		index_svit++;
	if (index_svit == INDEX_BATT2)
		index_svit++;
	
	currentValue = read_VIT_helper(SVIT[index_svit].Imux, SVIT[index_svit].ImuxBit);
	voltageValue = read_VIT_helper(SVIT[index_svit].Vmux, SVIT[index_svit].VmuxBit);
	if (SVIT[index_svit].Tmux != MUX_NULL)
		tempValue = read_VIT_helper(SVIT[index_svit].Tmux, SVIT[index_svit].TmuxBit);
		
	batt1CurrentValue = read_VIT_helper(SVIT[INDEX_BATT1].Imux, SVIT[INDEX_BATT1].ImuxBit);
	batt1VoltageValue = read_VIT_helper(SVIT[INDEX_BATT1].Vmux, SVIT[INDEX_BATT1].VmuxBit);
	batt1TempValue = read_VIT_helper(SVIT[INDEX_BATT1].Tmux, SVIT[INDEX_BATT1].TmuxBit);
	
	batt2CurrentValue = read_VIT_helper(SVIT[INDEX_BATT2].Imux, SVIT[INDEX_BATT2].ImuxBit);
	batt2VoltageValue = read_VIT_helper(SVIT[INDEX_BATT2].Vmux, SVIT[INDEX_BATT2].VmuxBit);
	batt2TempValue = read_VIT_helper(SVIT[INDEX_BATT2].Tmux, SVIT[INDEX_BATT2].TmuxBit);
}

// MUX_NUM is from {0, 1, 2} for the analog multiplexor that is being sampled from
// MUX_SEL is 0-31 for the input of the MUX that is selected and sampled

// sets the ADC registers of the MCU and actually performs the analog-to-digital conversion
unsigned int read_VIT_helper(char MUX_NUM, char MUX_SEL)
{
	unsigned int temp;
	selectMUX(MUX_NUM, MUX_SEL);
	ADMUX = (1 << REFS0) | MUX_NUM;	// sets voltage ref. to AVCC with external capacitor at AREF 
									// and selects one of the 3 ADC sampling channels
	ADCSRA |= (1 << ADSC);	// start conversion
	while (ADCSRA & (1 << ADSC));	// blocking while waiting for end of conversion
	temp = ADCL;	// we must read ADCL first according to spec sheet
	temp |= (ADCH << 8);	// reading ADCH will refresh ADC register
	return temp;
}

// drives the MCU port pins, connected to the select inputs of the activated MUX,
// with MUX_SEL depending on MUX_NUM, which indicates the activated MUX
/* #define MUX_SELECT(bit, port, pbit) ((bit) ? (port |= (1 << pbit)) : (port &= ~(1 << pbit))) OLD */
void selectMUX(char MUX_NUM, char MUX_SEL)
{
	if (MUX_NUM == 0)
	{
		//PORTA &= 0xf8;
		if(READ(MUX_SEL, 0)) SET(PORTA, PA3);
		else CLR(PORTA, PA3);
		if(READ(MUX_SEL, 1)) SET(PORTA, PA4);
		else CLR(PORTA, PA4);
		if(READ(MUX_SEL, 2)) SET(PORTA, PA5);
		else CLR(PORTA, PA5);
		if(READ(MUX_SEL, 3)) SET(PORTA, PA6);
		else CLR(PORTA, PA6);
		if(READ(MUX_SEL, 4)) SET(PORTA, PA7);
		else CLR(PORTA, PA7);
	}
	else if (MUX_NUM == 1)
	{
		//PORTE &= 0xf8;
		if(READ(MUX_SEL, 0)) SET(PORTE, PE7);
		else CLR(PORTE, PE7);
		if(READ(MUX_SEL, 1)) SET(PORTE, PE6);
		else CLR(PORTE, PE6);
		if(READ(MUX_SEL, 2)) SET(PORTE, PE5);
		else CLR(PORTE, PE5);
		if(READ(MUX_SEL, 3)) SET(PORTE, PE4);
		else CLR(PORTE, PE4);
		if(READ(MUX_SEL, 4)) SET(PORTE, PE3);
		else CLR(PORTE, PE3);
	}
	else if (MUX_NUM == 2)
	{
		//PORTB &= 0x1f;
		if(READ(MUX_SEL, 0)) SET(PORTB, PB4);
		else CLR(PORTB, PB4);
		if(READ(MUX_SEL, 1)) SET(PORTB, PB3);
		else CLR(PORTB, PB3);
		if(READ(MUX_SEL, 2)) SET(PORTB, PB2);
		else CLR(PORTB, PB2);
		if(READ(MUX_SEL, 3)) SET(PORTB, PB1);
		else CLR(PORTB, PB1);
		if(READ(MUX_SEL, 4)) SET(PORTB, PB0);
		else CLR(PORTB, PB0);
	}
}

/*	TOP LEVEL FUNCTION THAT SCALES THE ADC CONVERSION'S OUTPUT AND APPLIES
	A FILTER TO THIS SCALED VALUE BEFORE STORING IT IN THE APPROPRIATE 
	ELEMENT OF THE SVIT TABLE	*/
void storeValue()
{
	currentScaledValue = currentValue * SVIT[index_svit].ScaleFactorI;
	SVIT[index_svit].I = setValue(&I_SAMPLE[index_svit], currentScaledValue, 0);
	voltageScaledValue = voltageValue * SVIT[index_svit].ScaleFactorV;
	SVIT[index_svit].V = setValue(&V_SAMPLE[index_svit], voltageScaledValue, 1);
	if (SVIT[index_svit].Tmux != MUX_NULL)
	{
		tempScaledValue = tempValue * TempScaleFactor;
		SVIT[index_svit].T = setValue(&T_SAMPLE[index_temp++], tempScaledValue, 2);
		if (index_temp >= TEMP_SIZE) index_temp = 0;
	}
	
	batt1CurrentScaled = batt1CurrentValue * SVIT[INDEX_BATT1].ScaleFactorI;
	SVIT[INDEX_BATT1].I = setValue(&I_SAMPLE[INDEX_BATT1], batt1CurrentScaled, 0);
	batt1VoltageScaled = batt1VoltageValue * SVIT[INDEX_BATT1].ScaleFactorV;
	SVIT[INDEX_BATT1].V = setValue(&V_SAMPLE[INDEX_BATT1], batt1VoltageScaled, 1);
	batt1TempScaled = batt1TempValue * TempScaleFactor;
	SVIT[INDEX_BATT1].T = setValue(&T_SAMPLE[INDEX_BATT1], batt1TempScaled, 2);
	
	batt2CurrentScaled = batt2CurrentValue * SVIT[INDEX_BATT2].ScaleFactorI;
	SVIT[INDEX_BATT2].I = setValue(&I_SAMPLE[INDEX_BATT2], batt2CurrentScaled, 0);
	batt2VoltageScaled = batt2VoltageValue * SVIT[INDEX_BATT2].ScaleFactorV;
	SVIT[INDEX_BATT2].V = setValue(&V_SAMPLE[INDEX_BATT2], batt1VoltageScaled, 1);
	batt2TempScaled = batt2TempValue * TempScaleFactor;
	SVIT[INDEX_BATT2].T = setValue(&T_SAMPLE[INDEX_BATT2], batt2TempScaled, 2);
}

/* NOTE: SVIT array's I/V/T are not valid SMA values until after 8 executions of setValue() */

// returns the newly computed SMA of the most recent 8 samples (including the one from the previous read_VIT() call).
// Assumes that SVIT array's stored value is the last computed SMA.
// Updates 1 of the 8 samples stored in the SAMPLES array of the COMPONENT_t struct.
// ivtFlag: 0 = current, 1 = voltage, 2 = temperature
int setValue(COMPONENT_t * c, unsigned int v, char ivtFlag)
{	
	int newStoredValue, oldStoredValue;
	if (ivtFlag == 0) oldStoredValue = SVIT[index_svit].I;
	else if (ivtFlag == 1) oldStoredValue = SVIT[index_svit].V;
	else if (ivtFlag == 2) oldStoredValue = SVIT[index_svit].T;
	newStoredValue = oldStoredValue + ((v - (c -> SAMPLE[c -> S_INDEX])) >> 3);
	//c -> SAMPLE[c -> S_INDEX] = v;	// the dropped sample is replaced by the most recently taken sample
	//c -> S_INDEX++;
	if (c -> S_INDEX > 7)
		c -> S_INDEX = 0;
	return newStoredValue;
}

/* VALUE_RET function carried over from old power.c */
unsigned int retValue(COMPONENT_t * c)
{
	unsigned int ret, i;
	ret = 0;
	for (i = 0; i < 8; i++)
	{
		ret += c -> SAMPLE[i];
	}
	return (ret >> 3);
}

// updates the batter SOC variables in the SVIT struct
// if SOC(Vcell) is a monotonically decreasing function, then
//	x^n/(k-x^n) can model the curve well
void updateSOC(BATTERY_t * batt)
{
	unsigned int x = retValue(&batt -> METHOD0);
	unsigned int y = retValue(&batt -> METHOD1);
	batt -> TEMPERATURE = getTemp( retValue(&batt -> TEMP_SAMPLE) );
	if (x > y)
		batt -> SOC = (batt -> SOC) + x;
	else
		batt -> SOC = (batt -> SOC) + y;
}


// voltage-to-temperature conversion array stored in Flash memory
prog_int16_t ADC2TEMP[len] = { 
12498	,
12269	,
12056	,
11858	,
11673	,
11498	,
11334	,
11178	,
11030	,
10889	,
10755	,
10626	,
10503	,
10384	,
10271	,
10161	,
10055	,
9953	,
9854	,
9759	,
9666	,
9576	,
9489	,
9404	,
9321	,
9241	,
9162	,
9086	,
9011	,
8939	,
8868	,
8798	,
8730	,
8664	,
8599	,
8535	,
8472	,
8411	,
8351	,
8292	,
8234	,
8177	,
8122	,
8067	,
8013	,
7960	,
7908	,
7856	,
7806	,
7756	,
7707	,
7659	,
7611	,
7565	,
7518	,
7473	,
7428	,
7384	,
7340	,
7297	,
7254	,
7212	,
7171	,
7130	,
7090	,
7050	,
7010	,
6971	,
6932	,
6894	,
6857	,
6819	,
6782	,
6746	,
6710	,
6674	,
6639	,
6604	,
6569	,
6535	,
6501	,
6467	,
6434	,
6401	,
6368	,
6336	,
6304	,
6272	,
6240	,
6209	,
6178	,
6147	,
6117	,
6087	,
6057	,
6027	,
5998	,
5969	,
5940	,
5911	,
5883	,
5854	,
5826	,
5798	,
5771	,
5743	,
5716	,
5689	,
5662	,
5636	,
5609	,
5583	,
5557	,
5531	,
5505	,
5480	,
5455	,
5429	,
5404	,
5380	,
5355	,
5330	,
5306	,
5282	,
5258	,
5234	,
5210	,
5186	,
5163	,
5139	,
5116	,
5093	,
5070	,
5047	,
5025	,
5002	,
4980	,
4958	,
4935	,
4913	,
4891	,
4870	,
4848	,
4826	,
4805	,
4784	,
4762	,
4741	,
4720	,
4699	,
4678	,
4658	,
4637	,
4617	,
4596	,
4576	,
4556	,
4536	,
4516	,
4496	,
4476	,
4456	,
4437	,
4417	,
4398	,
4378	,
4359	,
4340	,
4321	,
4302	,
4283	,
4264	,
4245	,
4226	,
4208	,
4189	,
4171	,
4152	,
4134	,
4116	,
4097	,
4079	,
4061	,
4043	,
4025	,
4008	,
3990	,
3972	,
3955	,
3937	,
3919	,
3902	,
3885	,
3867	,
3850	,
3833	,
3816	,
3799	,
3782	,
3765	,
3748	,
3731	,
3715	,
3698	,
3681	,
3665	,
3648	,
3632	,
3615	,
3599	,
3583	,
3566	,
3550	,
3534	,
3518	,
3502	,
3486	,
3470	,
3454	,
3438	,
3422	,
3407	,
3391	,
3375	,
3360	,
3344	,
3329	,
3313	,
3298	,
3282	,
3267	,
3252	,
3236	,
3221	,
3206	,
3191	,
3176	,
3161	,
3146	,
3131	,
3116	,
3101	,
3086	,
3071	,
3057	,
3042	,
3027	,
3013	,
2998	,
2984	,
2969	,
2954	,
2940	,
2926	,
2911	,
2897	,
2883	,
2868	,
2854	,
2840	,
2826	,
2811	,
2797	,
2783	,
2769	,
2755	,
2741	,
2727	,
2713	,
2699	,
2686	,
2672	,
2658	,
2644	,
2630	,
2617	,
2603	,
2589	,
2576	,
2562	,
2549	,
2535	,
2521	,
2508	,
2495	,
2481	,
2468	,
2454	,
2441	,
2428	,
2414	,
2401	,
2388	,
2375	,
2361	,
2348	,
2335	,
2322	,
2309	,
2296	,
2283	,
2270	,
2257	,
2244	,
2231	,
2218	,
2205	,
2192	,
2179	,
2341	,
2332	,
2324	,
2315	,
2307	,
2299	,
2290	,
2282	,
2273	,
2265	,
2257	,
2249	,
2240	,
2232	,
2224	,
2215	,
2207	,
2199	,
2191	,
2183	,
2174	,
2166	,
2158	,
2150	,
2142	,
2134	,
2126	,
2117	,
2109	,
2101	,
2093	,
2085	,
2077	,
2069	,
2061	,
2053	,
2045	,
2037	,
2029	,
2021	,
2013	,
2005	,
1997	,
1989	,
1981	,
1973	,
1966	,
1958	,
1950	,
1942	,
1934	,
1926	,
1918	,
1911	,
1903	,
1895	,
1887	,
1879	,
1871	,
1864	,
1856	,
1848	,
1840	,
1833	,
1825	,
1817	,
1809	,
1802	,
1794	,
1786	,
1779	,
1771	,
1763	,
1755	,
1748	,
1740	,
1733	,
1725	,
1717	,
1710	,
1702	,
1694	,
1687	,
1679	,
1672	,
1664	,
1656	,
1649	,
1641	,
1634	,
1626	,
1618	,
1611	,
1603	,
1596	,
1588	,
1581	,
1573	,
1566	,
1558	,
1551	,
1543	,
1536	,
1528	,
1521	,
1513	,
1506	,
1498	,
1491	,
1483	,
1476	,
1469	,
1461	,
1454	,
1446	,
1439	,
1431	,
1424	,
1416	,
1409	,
1402	,
1394	,
1387	,
1379	,
1372	,
1365	,
1357	,
1350	,
1343	,
1335	,
1328	,
1320	,
1313	,
1306	,
1298	,
1291	,
1284	,
1276	,
1269	,
1262	,
1254	,
1247	,
1240	,
1232	,
1225	,
1218	,
1210	,
1203	,
1196	,
1188	,
1181	,
1174	,
1166	,
1159	,
1152	,
1144	,
1137	,
1130	,
1123	,
1115	,
1108	,
1101	,
1093	,
1086	,
1079	,
1072	,
1064	,
1057	,
1050	,
1042	,
1035	,
1028	,
1021	,
1013	,
1006	,
999	,
991	,
984	,
977	,
970	,
962	,
955	,
948	,
941	,
933	,
926	,
919	,
912	,
904	,
897	,
890	,
882	,
875	,
868	,
861	,
853	,
846	,
839	,
832	,
824	,
817	,
810	,
803	,
795	,
788	,
781	,
773	,
766	,
759	,
752	,
744	,
737	,
730	,
723	,
715	,
708	,
701	,
693	,
686	,
679	,
672	,
664	,
657	,
650	,
642	,
635	,
628	,
620	,
613	,
606	,
599	,
591	,
584	,
577	,
569	,
562	,
555	,
547	,
540	,
533	,
525	,
518	,
511	,
503	,
496	,
489	,
481	,
474	,
467	,
459	,
452	,
445	,
437	,
430	,
422	,
415	,
408	,
400	,
393	,
385	,
378	,
371	,
363	,
356	,
348	,
341	,
334	,
326	,
319	,
311	,
304	,
296	,
289	,
282	,
274	,
267	,
259	,
252	,
244	,
237	,
229	,
222	,
214	,
207	,
199	,
192	,
184	,
177	,
169	,
162	,
154	,
146	,
139	,
131	,
124	,
116	,
109	,
101	,
93	,
86	,
78	,
71	,
63	,
55	,
48	,
40	,
32	,
25	,
17	,
9	,
2	,
-6	,
-14	,
-21	,
-29	,
-37	,
-44	,
-52	,
-60	,
-68	,
-75	,
-83	,
-91	,
-99	,
-107	,
-114	,
-122	,
-130	,
-138	,
-146	,
-153	,
-161	,
-169	,
-177	,
-185	,
-193	,
-201	,
-209	,
-216	,
-224	,
-232	,
-240	,
-248	,
-256	,
-264	,
-272	,
-280	,
-288	,
-296	,
-304	,
-312	,
-320	,
-328	,
-336	,
-344	,
-353	,
-361	,
-369	,
-377	,
-385	,
-393	,
-401	,
-410	,
-418	,
-426	,
-434	,
-442	,
-451	,
-459	,
-467	,
-475	,
-484	,
-492	,
-500	,
-509	,
-517	,
-525	,
-534	,
-542	,
-550	,
-559	,
-567	,
-576	,
-584	,
-593	,
-601	,
-609	,
-618	,
-627	,
-635	,
-644	,
-652	,
-661	,
-669	,
-678	,
-687	,
-695	,
-704	,
-713	,
-721	,
-730	,
-739	,
-747	,
-756	,
-765	,
-774	,
-783	,
-791	,
-800	,
-809	,
-818	,
-827	,
-836	,
-845	,
-854	,
-863	,
-872	,
-881	,
-890	,
-899	,
-908	,
-917	,
-926	,
-935	,
-944	,
-954	,
-963	,
-972	,
-981	,
-990	,
-1000	,
-1009	,
-1018	,
-1028	,
-1037	,
-1046	,
-1056	,
-1065	,
-1075	,
-1084	,
-1094	,
-1103	,
-1113	,
-1122	,
-1132	,
-1142	,
-1151	,
-1161	,
-1171	,
-1180	,
-1190	,
-1200	,
-1210	,
-1220	,
-1229	,
-1239	,
-1249	,
-1259	,
-1269	,
-1279	,
-1289	,
-1299	,
-1309	,
-1319	,
-1329	,
-1340	,
-1350	,
-1360	,
-1370	,
-1381	,
-1391	,
-1401	,
-1412	,
-1422	,
-1433	,
-1443	,
-1454	,
-1464	,
-1475	,
-1485	,
-1496	,
-1507	,
-1517	,
-1528	,
-1539	,
-1550	,
-1561	,
-1571	,
-1582	,
-1593	,
-1604	,
-1615	,
-1626	,
-1638	,
-1649	,
-1660	,
-1671	,
-1682	,
-1694	,
-1705	,
-1717	,
-1728	,
-1739	,
-1751	,
-1763	,
-1774	,
-1786	,
-1798	,
-1809	,
-1821	,
-1833	,
-1845	,
-1857	,
-1869	,
-1881	,
-1893	,
-1905	,
-1917	,
-1930	,
-1942	,
-1954	,
-1967	,
-1979	,
-1992	,
-2004	,
-2017	,
-2029	,
-2042	,
-2055	,
-2068	,
-2081	,
-2094	,
-2107	,
-2120	,
-2133	,
-2146	,
-2159	,
-2173	,
-2186	,
-2200	,
-2213	,
-2227	,
-2240	,
-2254	,
-2268	,
-2282	,
-2296	,
-2310	,
-2324	,
-2338	,
-2352	,
-2367	,
-2381	,
-2396	,
-2410	,
-2425	,
-2440	,
-2454	,
-2469	,
-2484	,
-2499	,
-2515	,
-2530	,
-2545	,
-2561	,
-2576	,
-2592	,
-2607	,
-2623	,
-2639	,
-2655	,
-2671	,
-2688	,
-2704	,
-2720	,
-2737	,
-2754	,
-2770	,
-2787	,
-2804	,
-2821	,
-2839	,
-2856	,
-2873	,
-2891	,
-2909	,
-2927	,
-2945	,
-2963	,
-2981	,
-3000	,
-3018	,
-3037	,
-3056	,
-3075	,
-3094	,
-3113	,
-3133	,
-3152	,
-3172	,
-3192	,
-3212	,
-3233	,
-3253	,
-3274	,
-3295	,
-3316	,
-3337	,
-3359	,
-3380	,
-3402	,
-3424	,
-3447	,
-3469	,
-3492	,
-3515	,
-3538	,
-3562	,
-3586	,
-3610	,
-3634	,
-3658	,
-3683	,
-3708	,
-3734	,
-3759	,
-3785	,
-3811	,
-3838	,
-3865	,
-3892	,
-3920	,
-3948	,
-3976	,
-4005	,
-4034	,
-4063	,
-4093	,
-4124	,
-4155	,
-4186	,
-4217	,
-4250	,
-4282	,
-4315	,
-4349	,
-4383	,
-4418	,
-4454	,
-4490	,
-4526	,
-4564	,
-4602	,
-4641	,
-4680	,
-4720	,
-4761	,
-4803	,
-4846	,
-4890	,
-4934	,
-4980	,
-5027	,
-5075	,
-5124	,
-5174	,
-5225	,
-5278	,
-5332	,
-5388	
};

// returns the temperature value corresponding to the voltage input from ADC2TEMP array
int getTemp(int voltage)
{
	if (voltage < starting_index || voltage > ending_index)
	{
		return -1;
	}
	return pgm_read_word(((uint16_t)(ADC2TEMP)) + (voltage - starting_index)*2);
}
