#include "sun_packet.h"

/*
 * getAutomatic_Exposure_Control_ch0
 *
 * This function parses the gocode field of ch0 for
 * the automatic exposure control setting
 */
sint8 getAutomatic_Exposure_Control_ch0(sun_ch0 *ch0)
{
	return (sint8)(ch0->gocode & 0x01);
}

/*
 * getExposure_ch0
 *
 * This function parses the gocode field of ch0 for
 * the exposure setting
 */
sint8 getExposure_ch0(sun_ch0 *ch0)
{
	return (sint8)((ch0->gocode & 0x02) >> 1);
}

/*
 * getDark_Subtract_ch0
 *
 * This function parses the gocode field of ch0 for
 * the dark subtract setting
 */
sint8 getDark_Subtract_ch0(sun_ch0 *ch0)
{
	return (sint8)((ch0->gocode & 0x04) >> 2);
}

/*
 * getReadout_ch0
 *
 * This function parses the gocode field of ch0 for
 * the readout setting
 */
sint8 getReadout_ch0(sun_ch0 *ch0)
{
	return (sint8)((ch0->gocode & 0x08) >> 3);
}

/*
 * getCopy_To_Offline_ch0
 *
 * This function parses the gocode field of ch0 for
 * the copy to offline setting
 */
sint8 getCopy_To_Offline_ch0(sun_ch0 *ch0)
{
	return (sint8)((ch0->gocode & 0x10) >> 4);
}

/*
 * getCopy_From_Offline_ch0
 *
 * This function parses the gocode field of ch0 for
 * the copy from offline setting
 */
sint8 getCopy_From_Offline_ch0(sun_ch0 *ch0)
{
	return (sint8)((ch0->gocode & 0x20) >> 5);
}

/*
 * getFit_ch0
 *
 * This function parses the gocode field of ch0 for
 * the fit setting
 */
sint8 getFit_ch0(sun_ch0 *ch0)
{
	return (sint8)((ch0->gocode & 0x40) >> 6);
}


/*
 * The following are debugging functions, they print
 * the values of each channel's field to a file pointer
 */
void print_sun_ch0(sun_ch0 *ch0)
{
	printf("automatic exposure control: %d\n", getAutomatic_Exposure_Control_ch0(ch0));
	printf("exposure: %d\n", getExposure_ch0(ch0));
	printf("dark subtract: %d\n", getDark_Subtract_ch0(ch0));
	printf("readout: %d\n", getReadout_ch0(ch0));
	printf("copy to offline: %d\n", getCopy_To_Offline_ch0(ch0));
	printf("copy from offline: %d\n", getCopy_From_Offline_ch0(ch0));
	printf("fit: %d\n", getFit_ch0(ch0));
}

void print_sun_ch1(sun_ch1 *ch1)
{
	printf("min exposure: %d\n", ch1->min_exposure);
	printf("max exposure: %d\n", ch1->max_exposure);
	printf("target brightness: %d\n", ch1->target_brightness);
}

void print_sun_ch2(sun_ch2 *ch2)
{
	printf("min brightness: %d\n", ch2->min_brightness);
	printf("max brightness: %d\n", ch2->max_brightness);
	printf("integrated brightness: %d\n", ch2->integrated_brightness);
	printf("exposure length: %d\n", ch2->exposure_length);
}

void print_sun_ch3(sun_ch3 *ch3)
{
	printf("temperature: %d\n", ch3->temperature);
}

// How can you print an image to a string?
void print_sun_ch4_5(sun_ch4_5 *ch4_5)
{

}

void print_sun_ch6(sun_ch6 *ch6)
{
	printf("x vector: %f\n", ch6->x_vector);
	printf("y vector: %f\n", ch6->y_vector);
	printf("z vector: %f\n", ch6->z_vector);
	printf("fit quality: %d\n", (int)ch6->fit_quality);
	printf("geometry quality: %d\n", (int)ch6->geometry_quality);
}

void print_sun_ch7(sun_ch7 *ch7)
{
	printf("iterations: %d\n", (int)ch7->iterations);
}

void print_sun_ch8(sun_ch8 *ch8)
{
	printf("processing time: %d\n", ch8->processing_time);
}

void print_sun_ch9(sun_ch9 *ch9)
{
	printf("x vector: %d\n", (int)ch9->x_vector);
	printf("y vector: %d\n", (int)ch9->y_vector);
	printf("z vector: %d\n", (int)ch9->z_vector);
	printf("fit quality: %d\n", (int)ch9->fit_quality);
	printf("geometry quality: %d\n", (int)ch9->geometry_quality);
}

/*
 * getCommand_Code_sun
 *
 * This function retreives the value of the bits
 * that are responsible for the command code from
 * the message control field.
 */
uint8 getCommand_Code_sun(sun_packet *p)
{
	return p->message_control_field & 0x1F;
}

/*
 * getAck_sun
 *
 * This function retreives the value of the ACK bit
 * from the message control field.
 */
sint8 getAck_sun(sun_packet *p)
{
	return (p->message_control_field & 0x20) >> 5;
}

/*
 * getB_sun
 *
 * This function retreives the value of the B bit
 * from the message control field.
 */
sint8 getB_sun(sun_packet *p)
{
	return (p->message_control_field & 0x40) >> 6;
}

/*
 * getPoll_sun
 *
 * This function retreives the value of the Poll bit
 * from the message control field.
 */
sint8 getPoll_sun(sun_packet *p)
{
	return (p->message_control_field & 0x80) >> 7;
}

/*
 * The following functions are to aid in the parsing
 * of the sun packet's data field
 */
sun_ch0 *getCh0_sun(sun_packet *p)
{
	return (sun_ch0*)p->data;
}
sun_ch1 *getCh1_sun(sun_packet *p)
{
	return (sun_ch1*)p->data;
}
sun_ch2 *getCh2_sun(sun_packet *p)
{
	return (sun_ch2*)p->data;
}
sun_ch3 *getCh3_sun(sun_packet *p)
{
	return (sun_ch3*)p->data;
}
sun_ch4_5 *getCh4_5_sun(sun_packet *p)
{
	return (sun_ch4_5*)p->data;
}
sun_ch6 *getCh6_sun(sun_packet *p)
{
	return (sun_ch6*)p->data;
}
sun_ch7 *getCh7_sun(sun_packet *p)
{
	return (sun_ch7*)p->data;
}
sun_ch8 *getCh8_sun(sun_packet *p)
{
	return (sun_ch8*)p->data;
}
sun_ch9 *getCh9_sun(sun_packet *p)
{
	return (sun_ch9*)p->data;
}

/*
 * print_sun_packet
 *
 * This prints out all of the fields of the sun packet
 * including all of the fields of the data, regardless of the
 * channel.
 */
void print_sun_packet(sun_packet *p)
{
	printf("command code: %d\n", getCommand_Code_sun(p));
	printf("ack: %d\n", getAck_sun(p));
	printf("b: %d\n", getB_sun(p));
	printf("poll: %d\n", getPoll_sun(p));
	printf("channel: %d\n", p->channel);

	if (p->channel == 0)
		print_sun_ch0(getCh0_sun(p));
	else if (p->channel == 1)
		print_sun_ch1(getCh1_sun(p));
	else if (p->channel == 2)
		print_sun_ch2(getCh2_sun(p));
	else if (p->channel == 3)
		print_sun_ch3(getCh3_sun(p));
	else if (p->channel == 4 || p->channel == 5)
		print_sun_ch4_5(getCh4_5_sun(p));
	else if (p->channel == 6)
		print_sun_ch6(getCh6_sun(p));
	else if (p->channel == 7)
		print_sun_ch7(getCh7_sun(p));
	else if (p->channel == 8)
		print_sun_ch8(getCh8_sun(p));
	else if (p->channel == 9)
		print_sun_ch9(getCh9_sun(p));
}

/*
* Generates a NSP Protocol packet to send telecommands to a Sinclair device.
*
* Variables:
* dest_addr (8 bits) -> the address of the Sinclair device
* src_addr (8 bits) -> the address of the sender
* poll (1 bit) -> whether or not to expect data in return
* command_code (8 bits) -> the command to be executed
* data (0 - 2040 bits) -> the input/output data for/from that command
* length (8 bits) -> number of bytes of data (0 - 255)
*/
sint32 nsp_packet(uint8 dest_addr, uint8 src_addr, sint8 poll, uint8 command_code, uint8 channel, uint8 *data, uint8 length, uint8 *message_buf)
{
	message_buf[0] = dest_addr;
	message_buf[1] = src_addr;

	// Message Control Field
	message_buf[2] = (uint8)(128*poll | command_code);

	// Channel
	message_buf[3] = channel;

	// Data and CRC
	uint16 crc = 0xffff;

	uint8 *bufp = message_buf, ch;
	uint32 i, j;
	for (i = 0; i < (uint32)(4+length); i++)
	{
		if (i >= 4)
			message_buf[i] = data[i-4];

		ch = *bufp++;
		for (j = 0; j < 8; j++)
		{
			crc = (crc >> 1) ^ (((ch ^ crc) & 0x01) ? 0x8408 : 0);
			ch >>= 1;
        }
	}

	message_buf[4+length] = (uint8)(crc & 0x00FF);
	message_buf[5+length] = (uint8)((crc & 0xFF00) >> 8);
	message_buf[6+length] = '\r';

	return (sint32)(length+6);
}
