#include "common.h"

#ifndef sun_packet_h
#define sun_packet_h

// Sun Sensor Command Code Defines
#define SUN_PING					 0x00000000
#define SUN_INIT					 0x00000001
#define SUN_PEEK					 0x00000002
#define SUN_POKE					 0x00000003
#define SUN_TELEMETRY				 0x00000004
#define SUN_CRC			 			 0x00000006
#define SUN_APPLICATION_TELEMETRY    0x00000007
#define SUN_APPLICATION_COMMAND		 0x00000008

/*
 * sun_ch0
 *
 * Used to interpret a Channel 0 sun packet
 */
typedef struct
{
	uint8 gocode;
} __attribute__((__packed__)) sun_ch0;

// look in sun_packet.c for documentation for the following functions
sint8 getAutomatic_Exposure_Control_ch0(sun_ch0 *ch0);
sint8 getExposure_ch0(sun_ch0 *ch0);
sint8 getDark_Subtract_ch0(sun_ch0 *ch0);
sint8 getReadout_ch0(sun_ch0 *ch0);
sint8 getCopy_To_Offline_ch0(sun_ch0 *ch0);
sint8 getCopy_From_Offline_ch0(sun_ch0 *ch0);
sint8 getFit_ch0(sun_ch0 *ch0);
void print_sun_ch0(sun_ch0 *ch0);

/*
 * sun_ch1
 *
 * Used to interpret a Channel 1 sun packet
 */
typedef struct
{
	uint32 min_exposure;
	uint32 max_exposure;
	uint16 target_brightness;
} __attribute__((__packed__)) sun_ch1;

// look in sun_packet.c for documentation for the following functions
void print_sun_ch1(sun_ch1 *ch1);

/*
 * sun_ch2
 *
 * Used to interpret a Channel 2 sun packet
 */
typedef struct
{
	uint16 min_brightness;
	uint16 max_brightness;
	uint32 integrated_brightness;
	uint32 exposure_length;
} __attribute__((__packed__)) sun_ch2;

// look in sun_packet.c for documentation for the following functions
void print_sun_ch2(sun_ch2 *ch2);

/*
 * sun_ch3
 *
 * Used to interpret a Channel 3 sun packet
 */
typedef struct
{
	uint16 temperature;
} __attribute__((__packed__)) sun_ch3;

// look in sun_packet.c for documentation for the following functions
void print_sun_ch3(sun_ch3 *ch3);

/*
 * sun_ch4_5
 *
 * Used to interpret a channel 4 or channel 5 sun packet
 */
typedef struct
{
	uint16 image[256];
} __attribute__((__packed__)) sun_ch4_5;

// look in sun_packet.c for documentation for the following functions
void print_sun_ch4_5(sun_ch4_5 *ch4_5);

/*
 * sun_ch6
 *
 * Used to interpret a channel 6 sun packet
 */
typedef struct
{
	float32 x_vector;
	float32 y_vector;
	float32 z_vector;
	uint8 fit_quality;
	uint8 geometry_quality;
} __attribute__((__packed__)) sun_ch6;

// look in sun_packet.c for documentation for the following functions
void print_sun_ch6(sun_ch6 *ch6);

/*
 * sun_ch7
 *
 * Used to interpret a channel 7 sun packet
 */
typedef struct
{
	uint8 iterations;
} __attribute__((__packed__)) sun_ch7;

// look in sun_packet.c for documentation for the following functions
void print_sun_ch7(sun_ch7 *ch7);

/*
 * sun_ch8
 *
 * Used to interpret a channel 8 sun packet
 */
typedef struct
{
	uint32 processing_time;
} __attribute__((__packed__)) sun_ch8;

// look in sun_packet.c for documentation for the following functions
void print_sun_ch8(sun_ch8 *ch8);

/*
 * sun_ch9
 *
 * Used to interpret a channel 9 sun packet
 */
typedef struct
{
	uint16 x_vector;
	uint16 y_vector;
	uint16 z_vector;
	uint8 fit_quality;
	uint8 geometry_quality;
} __attribute__((__packed__)) sun_ch9;

// look in sun_packet.c for documentation for the following functions
void print_sun_ch9(sun_ch9 *ch9);

/*
 * sun_packet
 *
 * This is overall sun packet structure.
 * All of the channel structures are defined
 * in order to interpret the data field of this
 * structure. For example, if the channel field
 * is 0 then use the sun_ch0 structure.
 */
typedef struct
{
	uint8 message_control_field;

	uint8 channel;
	uint8 data[0];
} __attribute__((__packed__)) sun_packet;

// look in sun_packet.c for documentation for the following functions
uint8 getCommand_Code_sun(sun_packet *p);
sint8 getAck_sun(sun_packet *p);
sint8 getB_sun(sun_packet *p);
sint8 getPoll_sun(sun_packet *p);

sun_ch0 *getCh0_sun(sun_packet *p);
sun_ch1 *getCh1_sun(sun_packet *p);
sun_ch2 *getCh2_sun(sun_packet *p);
sun_ch3 *getCh3_sun(sun_packet *p);
sun_ch4_5 *getCh4_5_sun(sun_packet *p);
sun_ch6 *getCh6_sun(sun_packet *p);
sun_ch7 *getCh7_sun(sun_packet *p);
sun_ch8 *getCh8_sun(sun_packet *p);
sun_ch9 *getCh9_sun(sun_packet *p);

void print_sun_packet(sun_packet *p);

int nsp_packet(uint8 dest_addr, uint8 src_addr, sint8 poll, uint8 command_code, uint8 channel, uint8 *data, uint8 length, uint8 *message_buf);
// TODO add a function here to extract the data from an nsp packet

#endif
